/* scanner for CODE */

D [0-9]
L [a-zA-Z]
S [_]

%{

#include <stdio.h>
#include <stdlib.h>

#include "../misc/general.h"
#include "../exmodel/abstree.h"
#include "../exmodel/exmodel.h"
#include "parser.h"
#include "scan.h"
#include <string.h>

/* Define YY_INPUT so it reads from the string buffer.
   unput() is defined appropriately, below, to just
   move the pointer (bp) backwards. */

extern char * bp;
#define YY_INPUT(buf,result,max_size) { \
   char ch = (char) 1; \
   if ((ch = *bp)) { \
     ++bp; \
     buf[0] = ch; \
     result = 1; \
   } else { \
     result = YY_NULL; \
   } \
}


#ifndef YYSTYPE
typedef cAbsTree *yyMyType;
#define YYSTYPE yyMyType
#endif

#include "gram.h"


extern cAbsTree *yylval;     /* Value of token for yacc++ */

static char *cpstr(char *s);
static void count();

#ifndef yylex
        extern "C" int yylex (void);
#endif
#ifndef yyerror
        extern "C" void yyerror(const char *s);
#endif

#undef lex_input
#undef unput

%}

%%

"%{"(.|[\t\v\n\r\f])*"}%" {
	     count();
	     yylval = (cAbsTree *) cpstr(yytext+2);
	     return (CPL_DEFINITIONS);
           }

"%"			{ count(); return '%'; }
"!"			{ count(); return '!'; }
"-"			{ count(); return '-'; }
"["			{ count(); return '['; }
"]"			{ count(); return ']'; }
"<"			{ count(); return '<'; }
">"			{ count(); return '>'; }
"."			{ count(); return '.'; }
","			{ count(); return ','; }

"{"			{ count(); return '{'; }
"}"			{ count(); return '}'; }
";"			{ count(); return ';'; }
"("			{ count(); return '('; }
")"			{ count(); return ')'; }
"*"			{ count(); return '*'; }
"+"			{ count(); return '+'; }
"="			{ count(); return '='; }
":"			{ count(); return ':'; }
"/"			{ count(); return '/'; }

"->"			{ count(); return(CONS); }
"<-"			{ count(); return(PROD); }
"<="			{ count(); return(LE_OP); }
">="			{ count(); return(GE_OP); }
"=="			{ count(); return(EQ_OP); }
"!="			{ count(); return(NE_OP); }
"&&"			{ count(); return(AND_OP); }
"||"			{ count(); return(OR_OP); }

"new"			{ count(); return(NEW); }
"void"			{ count(); return(VOID); }
"if"			{ count(); return(IF); }
"else"			{ count(); return(ELSE); }
"while"			{ count(); return(WHILE); }
"is"			{ count(); return(IS); }
"of"			{ count(); return(OF); }
"shared_vars"		{ count(); return(SHARED_VAR); }
"variables"		{ count(); return(VAR); }
"type"			{ count(); return(TYPE); }
"array"			{ count(); return(ARRAY); }
"comp"			{ count(); return(COMP); }
"initial"		{ count(); return(INITCOMP); }
"fires"			{ count(); return(INPUTRULES); }
"routes"		{ count(); return(OUTPUTRULES); }
"input_ports"		{ count(); return(INPUTPORTS); }
"output_ports"		{ count(); return(OUTPUTPORTS); }
"reader"		{ count(); return(READER); }
"writer"		{ count(); return WRITER; }
"struct"		{ count(); return(STRUCT); }
"qqfuncsig"		{ count(); return(QQFUNCSIG); }
"qqarctopo"		{ count(); return(QQARCTOPO); }
"qquc"			{ count(); return(QQUC); }
"qqnsicomp"		{ count(); return(QQNSICOMP); }
"qqtypedef"		{ count(); return(QQTYPEDEF); }
"qqexpr"		{ count(); return(QQEXPR); }
"Partition"		{ count(); return(PARTITION); }
"Merge"		        { count(); return(MERGE); }
"Expand"		{ count(); return(EXPAND); }
"Contract"		{ count(); return(CONTRACT); }
"Strip-Update"		{ count(); return(STRIP_UPDATE); }
"Align"  		{ count(); return(ALIGN); }
"BLOCK"	         	{ count(); return(BLOCK); }
"CYCLIC"		{ count(); return(CYCLIC); }
"BLOCK_CYCLIC"		{ count(); return(BLOCK-CYCLIC); }
"OVERLAP"		{ count(); return(OVERLAP); }
"Summary"		{ count(); return(SUMMARY); }

"start"                 { count(); return(START); }
"stop"                  { count(); return(STOP); }
"static"                { count(); return(STATIC); }
"program"               { count(); return(CPL_PROGRAM); }
"functions"		{ count(); return(CPL_FUNCTION); }
"arcs"			{ count(); return(CPL_ARC); }
"typedef"		{ count(); return(CPL_TYPEDEF); }
"computation"		{ count(); return(CPL_UC); }
"namespace"		{ count(); return(CPL_NSICOMP); }
"container"		{ count(); return(CPL_GRAPH); }
"alias"                 { count(); return(CPL_CALL); }
"crep"			{ count(); return(CPL_CREP); }
"input"			{ count(); return(CPL_INPUT); }
"output"		{ count(); return(CPL_OUTPUT); }

{D}*\.{D}*		{ count(); 
			  yylval = new cRealConst(atof(yytext));
			  return(REALCONST); }

{D}+			{ count();
			  yylval = new cIntConst(atoi(yytext));
			  return(INTCONST); }

({L}|{S})({L}|{D}|{S})*	{ count();
			  yylval = (cAbsTree *) cpstr(yytext);
			  return(IDENT); }


"//".*			{ count();  /* ignore comments */ }
[ \t\v\n\r\f]		{ count(); }

.			{ count(); return(WRONG); }

%%

static int yyecho = 1;    /* if true produce listing to stdout */
int yycolumn;
int yylinenum;


static void count()
{
   int i;

   for (i = 0; yytext[i] != '\0'; i++)
     if (yytext[i] == '\n') {
       yylinenum += 1;
       yycolumn = 0;
    }
     else if (yytext[i] == '\t')
       yycolumn += 8 - (yycolumn % 8);
     else
       yycolumn++;
   
   if (yyecho) ECHO;

}


#ifndef yyerror
extern "C" void yyerror (const char *s)
{
   if (yyecho) {
      printf("\n ** At line %d, col %d... %s\n", yylinenum, yycolumn, s);
      fflush(stdout);
   }
}
#endif

int yywrap(void)
{
        return(1);
}

#include <assert.h>
#include <strings.h>

char *bp;
static char *bufstart;
static char *codebp;
static char *codebufstart;
static int InCodeString;
static char yyCode[32];

// Functions below are to allow the parser to read from this "string"
// consisting of the code string followed by the data.  Weird stuff.

void yyset_input(char *buf, int DoListing)
{
   extern int yyprevious;

   yycolumn = 0;
   yylinenum = 0;
   yyecho = DoListing;
   InCodeString = 0;

   codebp = buf;
   codebufstart = buf;

   bp = buf;
   bufstart = buf;
#if 0
   NLSTATE;         /* Tell parser it is at beginning of line */
#endif
}

#if 1
int lex_input()
{
   char ch;

   if (InCodeString) {
      ch = *codebp;
      ++codebp;
      if (*codebp == '\0') {
	 InCodeString = 0;
	 --codebp;
      }
   } else
     if (ch = *bp) ++bp;

   return ch;
}
#endif

void unput (int ch) {
  assert (bp > bufstart);
  --bp;
}

#if 0
unput(int ch)
{

   if (ch == '\0') return 0;

   if (InCodeString) {
      --codebp;
      if (*codebp != ch) Die("unput the wrong char in code!\n");
      if (codebp < codebufstart) Die("unput past starting point in code!\n");
   } else {
      if (bp > bufstart) {
	 --bp;
	 if (*bp != ch) Die("unput the wrong char!\n");
      } else
	InCodeString = 1;
   }

   return 1; /* Silence the "no return value" warning */
}
#endif



  // Copy string out of yytext into static storage.  This is yet another
  // memory leak since these are just copied into an MString, but there
  // is no easy way to fix it.

static char *cpstr(char *s)
{
   char* strbuf;

   strbuf = new char[strlen(s)+1];

   strcpy(strbuf, s);
   return strbuf;
}
