/*
  fsynch.c
  ------------------------------------------------------------------------
  Wrappers for POSIX synchronization variables.
  ------------------------------------------------------------------------
  @(#) $Id: fsynch.C,v 1.13 1999/05/10 21:15:48 emery Exp $
  ------------------------------------------------------------------------
  AUTHOR/CONTACT:
 
  Emery Berger                    | <http://www.cs.utexas.edu/users/emery>
  Parallel Programming Group      |  <http://www.cs.utexas.edu/users/code>
  Department of Computer Sciences |             <http://www.cs.utexas.edu>
  University of Texas at Austin   |                <http://www.utexas.edu>
  ========================================================================
*/


#include <stdio.h>
#include <time.h>

#include "fthread.h"
#include "fsynch.h"


void ConditionInit (Cond *cond) {
  pthread_cond_init (cond, NULL);
}

void LockInit (Lock *lock, ...) {
  pthread_mutex_init (lock, NULL);
}

int ConditionTimedWait (Cond * cond, Lock * lock, int sec)
{
  struct timespec abstime;
  abstime.tv_sec = time (NULL) + sec;
  abstime.tv_nsec = 0;
  return (pthread_cond_timedwait (cond, lock, &abstime));
}


int LockTryAcquire (Lock * lock)
{
  int result = pthread_mutex_trylock (lock);
  if (result) {
    printf ("Lock already acquired (%p): sleeptime\n", lock);
  }
  return result;
}
