/*
  port.C
  ------------------------------------------------------------------------
  Input port management.
  ------------------------------------------------------------------------
  @(#) $Id: port.C,v 1.17 2000/01/24 22:57:38 emery Exp $
  ------------------------------------------------------------------------
  AUTHOR/CONTACT:
 
  Emery Berger                    | <http://www.cs.utexas.edu/users/emery>
  Parallel Programming Group      |  <http://www.cs.utexas.edu/users/code>
  Department of Computer Sciences |             <http://www.cs.utexas.edu>
  University of Texas at Austin   |                <http://www.utexas.edu>
  ========================================================================
*/

#define _PTHREADS

#include <deque.h>
#include "port.H"
#include "sermach.H"


// A function object used for finding a matching entry.
class PortIndexEquals {
public:

  PortIndexEquals (Index portindex)
    : _portindex (portindex)
  {}

  int operator() (Port::Item& p)
  {
    return (p.portindex == _portindex);
  }

private:
  Index	_portindex;
};


Port::Port (void)
{
  for (int i = 0; i < MAX_INDICES; i++) {
    _pq[i] = new deque<Item, malloc_alloc>;
  }
}


void Port::Add (const Index& portindex,
		const _c2_Value& value)
{
  int n = hashindex(portindex);
  Guard m (_lock[n]);
  Item newitem (portindex, value);
  _pq[n]->push_back (newitem);
}


int Port::Find (Index& portindex)
{
  int n = hashindex (portindex);
  Guard m (const_cast(Port *, this)->_lock[n]);

  deque<Item, malloc_alloc>::iterator i;

  int j = _pq[n]->size();

  for (i = _pq[n]->begin ();
       i != _pq[n]->end ();
       ++i, --j) {
    if (portindex == (*i).portindex)
      break;
    assert (j >= 0);
  }

  if (i != _pq[n]->end ()) {
    return 1;
  }
  return 0;
}


int Port::Remove (Index& portindex,
		  _c2_Value& value)
{
  int n = hashindex (portindex);
  Guard m (_lock[n]);
  deque<Item, malloc_alloc>::iterator i;
  // Look for a matching port index.

  for (i = _pq[n]->begin ();
       i != _pq[n]->end ();
       ++i) {
    if (portindex == (*i).portindex)
      break;
  }

  if (i != _pq[n]->end ()) {
    value = (*i).value;
    _pq[n]->erase (i);
    return 1;
  }
  // No match found.
  return 0;
}
